;; Some functions for loading and saving games.

; Predicates for splitting the data table in to two parts - the part
; which has only plain data, and can be written to a file, and the
; part that has functions as data and can not be modified by the game
; after (init) is run.
(define (has-function pair)
  (procedure? (cdr pair)))
(define (not-function pair)
  (not (has-function pair)))

; Make a save file filename out of a symbol.
(define (make-filename name)
  (string-append (symbol->string name) ".save"))

; Save a game by writing the non-function part of the data tree to a
; file.
(define (save-game game-name)
  (let ((filename (make-filename game-name)))
    (if (file-exists? filename)
	(delete-file filename))
    (let ((out-file (open-output-file filename)))
      (write (filter-table not-function (get-data)) out-file)
      (close-output-port out-file))))

; Load a game by combining the function part of the current data tree
; with the non-function part of the tree in a file.
(define (load-game game-name)
  (let ((filename (make-filename game-name)))
    (cond ((not (file-exists? filename))
	   (print-message "sorry, could not find " filename ".")
	   #f)
	  (else
	   (let ((in-file (open-input-file filename)))
	     (set! data (filter-table has-function data))
	     (merge-table (read in-file) (get-data))
	     (close-input-port in-file)
	     #t)))))
